
#include "defaultgui.h"
#include "helper.h"
#include "core.h"
#include "global.h"
#include "timeslider.h"
#include "constants.h"
#include "playlist.h"
#include "mplayerwindow.h"

#include <qmainwindow.h>
#include <qlabel.h>
#include <qstatusbar.h>
#include <qmenubar.h>
#include "myaction.h"
#include <qtooltip.h>
#include <qpushbutton.h>


DefaultGui::DefaultGui( QString file, QWidget * parent, const char* name, 
                        WFlags fl )
	: BaseGui( file, parent, name, fl )
{

	createStatusBar();

	connect( this, SIGNAL(timeChanged(double, int, QString)),
             this, SLOT(displayTime(double, int, QString)) );
    connect( this, SIGNAL(frameChanged(int)),
             this, SLOT(displayFrame(int)) );

	connect( this, SIGNAL(cursorNearBottom(QPoint)), 
             this, SLOT(showFloatingControl(QPoint)) );
	connect( this, SIGNAL(cursorNearTop(QPoint)), 
             this, SLOT(showFloatingMenu(QPoint)) );
	connect( this, SIGNAL(cursorFarEdges()), 
             this, SLOT(hideFloatingControls()) );

	createMainToolBars();
    createControlWidget();
    createControlWidgetMini();

#if DOCK_PLAYLIST
	// Playlistdock
	playlistdock = new QDockWindow(QDockWindow::OutsideDock, this, "playlist" );
	playlist->reparent(playlistdock, QPoint(0,0), FALSE);
	playlistdock->setWidget( playlist);
	playlistdock->setResizeEnabled(TRUE);
	playlistdock->setCloseMode(QDockWindow::Always);
	playlistdock->setVerticallyStretchable(TRUE);
	addDockWindow(playlistdock, DockBottom );
	//setDockEnabled(Qt::DockBottom, TRUE);
	playlistdock->hide();
#endif

	languageChange();
}

DefaultGui::~DefaultGui() {
#if DOCK_PLAYLIST
	playlist->reparent(0, QPoint(0,0), FALSE);
#endif
}

void DefaultGui::createMainToolBars() {
	toolbar1 = new QToolBar( this, "toolbar1" );
	openFileAct->addTo( toolbar1 );
	openDVDAct->addTo( toolbar1 );
	openURLAct->addTo( toolbar1 );
	toolbar1->addSeparator();
	compactAct->addTo( toolbar1 );
	fullscreenAct->addTo( toolbar1 );
	toolbar1->addSeparator();
	screenshotAct->addTo( toolbar1 );
	toolbar1->addSeparator();
	showInfoAct->addTo( toolbar1 );
	showPlaylistAct->addTo( toolbar1 );
	showPreferencesAct->addTo( toolbar1 );

	toolbar2 = new QToolBar( this, "toolbar2" );
	select_audio = new QPushButton( toolbar2, "select_audio" );
	select_audio->setPopup( (QPopupMenu *) audiotrack_menu );

	select_subtitle = new QPushButton( toolbar2, "select_subtitle" );
	select_subtitle->setPopup( (QPopupMenu *) subtitlestrack_menu );

	toolbar1->show();
	toolbar2->show();
}


void DefaultGui::createControlWidgetMini() {
	controlwidget_mini = new QToolBar( this, "controlwidget" );
	controlwidget_mini->setResizeEnabled(false);
	controlwidget_mini->setMovingEnabled(false);
	addDockWindow(controlwidget_mini, Qt::DockBottom );

	//rewind3Act->addTo( controlwidget_mini );
	//rewind2Act->addTo( controlwidget_mini );
	rewind1Act->addTo( controlwidget_mini );

	timeslider_mini = new TimeSlider( controlwidget_mini, "timeslider_mini");
	connect( timeslider_mini, SIGNAL( posChanged(int) ), 
             core, SLOT(goToPos(int)) );
	connect( timeslider_mini, SIGNAL( draggingPos(int) ), 
             this, SLOT(displayGotoTime(int)) );
	controlwidget_mini->setStretchableWidget( timeslider_mini );

	forward1Act->addTo( controlwidget_mini );
	//forward2Act->addTo( controlwidget_mini );
	//forward3Act->addTo( controlwidget_mini );

	controlwidget_mini->addSeparator();

	muteAct->addTo( controlwidget_mini );
	volumeslider_mini = new MySlider( controlwidget_mini, "volumeslider_mini" );
	volumeslider_mini->setOrientation( Qt::Horizontal );
	volumeslider_mini->setSizePolicy( QSizePolicy::Fixed, QSizePolicy::Fixed );
	volumeslider_mini->setFocusPolicy( NOFOCUS );
	connect( volumeslider_mini, SIGNAL( valueChanged(int) ), 
             core, SLOT( setVolume(int) ) );

	controlwidget_mini->hide();
}

void DefaultGui::createControlWidget() {
	controlwidget = new QToolBar( this, "controlwidget" );
	controlwidget->setResizeEnabled(false);
	controlwidget->setMovingEnabled(false);
	addDockWindow(controlwidget, Qt::DockBottom );

	playAct->addTo( controlwidget );
	pauseAndStepAct->addTo( controlwidget );
	stopAct->addTo( controlwidget );

	controlwidget->addSeparator();

	rewind3Act->addTo( controlwidget );
	rewind2Act->addTo( controlwidget );
	rewind1Act->addTo( controlwidget );

	timeslider = new TimeSlider( controlwidget, "timeslider");
	connect( timeslider, SIGNAL( posChanged(int) ), 
             core, SLOT(goToPos(int)) );
	connect( timeslider, SIGNAL( draggingPos(int) ), 
             this, SLOT(displayGotoTime(int)) );
	controlwidget->setStretchableWidget( timeslider );

	forward1Act->addTo( controlwidget );
	forward2Act->addTo( controlwidget );
	forward3Act->addTo( controlwidget );

	controlwidget->addSeparator();

	fullscreenAct->addTo( controlwidget );
	muteAct->addTo( controlwidget );

	volumeslider = new MySlider( controlwidget, "volumeslider" );
	volumeslider->setOrientation( Qt::Horizontal );
	volumeslider->setSizePolicy( QSizePolicy::Fixed, QSizePolicy::Fixed );
	volumeslider->setFocusPolicy( NOFOCUS );
	volumeslider->setTickmarks( QSlider::Below );
	volumeslider->setTickInterval( 10 );
	volumeslider->setLineStep( 1 );
	volumeslider->setPageStep( 10 );
	connect( volumeslider, SIGNAL( valueChanged(int) ), 
             core, SLOT( setVolume(int) ) );

	controlwidget->show();
}



void DefaultGui::createStatusBar() {
	qDebug("DefaultGui::createStatusBar");

	time_display = new QLabel( statusBar() );
	time_display->setAlignment(Qt::AlignRight);

	frame_display = new QLabel( statusBar() );
	frame_display->setAlignment(Qt::AlignRight);
    #if QT_VERSION >= 0x040000
	statusBar()->setAutoFillBackground(TRUE);
	#endif
	statusBar()->setPaletteBackgroundColor( QColor(0,0,0) );
	statusBar()->setPaletteForegroundColor( QColor(255,255,255) );
	time_display->setPaletteBackgroundColor( QColor(0,0,0) );
	time_display->setPaletteForegroundColor( QColor(255,255,255) );
	frame_display->setPaletteBackgroundColor( QColor(0,0,0) );
	frame_display->setPaletteForegroundColor( QColor(255,255,255) );
	statusBar()->setSizeGripEnabled(FALSE);

    statusBar()->message( tr("Welcome to SMPlayer") );
	statusBar()->addWidget( frame_display, 0, TRUE );
	frame_display->setText( " 0 " );

    statusBar()->addWidget( time_display, 0, TRUE );
	time_display->setText("00:00:00 / 00:00:00");

	time_display->show();
	frame_display->hide();
}

void DefaultGui::languageChange() {
	BaseGui::languageChange();

	QToolTip::add( volumeslider, tr("Volume") );

	select_audio->setText( tr("Audio") );
	select_subtitle->setText( tr("Subtitle") );

#if DOCK_PLAYLIST
    playlistdock->setCaption( tr("Playlist") );
#endif
}

void DefaultGui::showPlaylist(bool b) {
	qDebug("DefaultGui::showPlaylist");
#if !DOCK_PLAYLIST
	BaseGui::showPlaylist(b);
#else
    if ( !b ) {
        playlistdock->hide();
    } else {
        exitFullscreenIfNeeded();
        playlistdock->show();
    }
    updateWidgets();
#endif
}


void DefaultGui::displayTime(double sec, int perc, QString text) {
	time_display->setText( text );
	timeslider->setPos(perc);
	timeslider_mini->setPos(perc);
}

void DefaultGui::displayFrame(int frame) {
	if (frame_display->isVisible()) {
		frame_display->setNum( frame );
	}
}

void DefaultGui::updateWidgets() {
	qDebug("DefaultGui::updateWidgets");

	BaseGui::updateWidgets();

	// Frame counter
	frame_display->setShown( pref->show_frame_counter );

	// Volume slider
	volumeslider->setValue( core->mset.volume );
	volumeslider_mini->setValue( core->mset.volume );
}

void DefaultGui::aboutToEnterFullscreen() {
	qDebug("DefaultGui::aboutToEnterFullscreen");

	if (!pref->compact_mode) {
		menuBar()->hide();
		statusBar()->hide();
		controlwidget->hide();
		controlwidget_mini->hide();
		toolbar1->hide();
		toolbar2->hide();
		#if DOCK_PLAYLIST
		playlistdock->hide();
		#endif
	}
}

void DefaultGui::aboutToExitFullscreen() {
	qDebug("DefaultGui::aboutToExitFullscreen");

	if (!pref->compact_mode) {
		menuBar()->show();
		statusBar()->show();
		controlwidget->show();
#if CONTROLWIDGET_OVER_VIDEO
		#if QT_VERSION >= 0x040000
		addDockWindow(controlwidget, Qt::DockBottom );
		#endif
#endif
		toolbar1->show();
		toolbar2->show();
		#if DOCK_PLAYLIST
		playlistdock->show();
		#endif
	}
}

void DefaultGui::aboutToEnterCompactMode() {
	menuBar()->hide();
	statusBar()->hide();
	controlwidget->hide();
	controlwidget_mini->hide();
	toolbar1->hide();
	toolbar2->hide();
}

void DefaultGui::aboutToExitCompactMode() {
	menuBar()->show();
	statusBar()->show();
	toolbar1->show();
	toolbar2->show();

	controlwidget->show();

	// Recheck size of controlwidget
	resizeEvent( new QResizeEvent( size(), size() ) );
}

void DefaultGui::showFloatingControl(QPoint p) {
	qDebug("DefaultGui::showFloatingControl");

#if CONTROLWIDGET_OVER_VIDEO
	//if (controlwidget->place() == QDockWindow::InDock) {
	if (!controlwidget->isVisible()) {
		controlwidget->undock();
		controlwidget->move(0, mplayerwindow->height()- controlwidget->height() );
		controlwidget->resize( mplayerwindow->width(), controlwidget->height() );
		controlwidget->show();
	}
	
#else
	if (!controlwidget->isVisible()) {
		controlwidget->show();
	}
#endif
}

void DefaultGui::showFloatingMenu(QPoint p) {
#if !CONTROLWIDGET_OVER_VIDEO
	qDebug("DefaultGui::showFloatingMenu");

	if (!menuBar()->isVisible())
		menuBar()->show();
#endif
}

void DefaultGui::hideFloatingControls() {
	qDebug("DefaultGui::hideFloatingControls");

#if CONTROLWIDGET_OVER_VIDEO
	//if (controlwidget->place() == QDockWindow::OutsideDock) {
	if (controlwidget->isVisible()) {
		controlwidget->dock();
		controlwidget->hide();
	}
#else
	if (controlwidget->isVisible())	
		controlwidget->hide();

	if (menuBar()->isVisible())	
		menuBar()->hide();
#endif
}

void DefaultGui::resizeEvent( QResizeEvent * ) {
	qDebug("defaultGui::resizeEvent");
	qDebug(" controlwidget width: %d", controlwidget->width() );
	qDebug(" controlwidget_mini width: %d", controlwidget_mini->width() );

#if QT_VERSION < 0x040000
#define LIMIT 470
#else
#define LIMIT 570
#endif

	if ( (controlwidget->isVisible()) && (width() < LIMIT) ) {
		controlwidget->hide();
		controlwidget_mini->show();
	}
	else
	if ( (controlwidget_mini->isVisible()) && (width() > LIMIT) ) {
		controlwidget_mini->hide();
		controlwidget->show();
	}
}
